import cocotb 

from cocotb.clock import Clock 
from cocotb.triggers import Timer, RisingEdge, FallingEdge

from cocotbext.axi import AxiStreamBus, AxiStreamSink, AxiStreamSource, AxiLiteMaster, AxiLiteBus
from cocotbext.eth import RgmiiPhy, GmiiFrame

from scapy.volatile import RandMAC
from scapy.layers.l2 import Ether

import logging

CLK_PERIOD_100 = 10
CLK_PERIOD_125 = 8


class TB:
    def __init__(self, dut):
        self.dut = dut

        self.log = logging.getLogger("cocotb.tb")
        self.log.setLevel(logging.INFO)

        cocotb.start_soon(Clock(dut.u_dut.i_clk_100, CLK_PERIOD_100, units="ns").start())
        cocotb.start_soon(Clock(dut.u_dut.i_clk_125, CLK_PERIOD_125, units="ns").start())

        # Slice indexing is not supported :(
        self.rgmii_0 = RgmiiPhy(self.dut.rgmii_txd_0, self.dut.rgmii_tx_ctl_0, self.dut.rgmii_tx_clk_0,
                                self.dut.rgmii_rxd_0, self.dut.rgmii_rx_ctl_0, self.dut.rgmii_rx_clk_0)

        self.rgmii_1 = RgmiiPhy(self.dut.rgmii_txd_1, self.dut.rgmii_tx_ctl_1, self.dut.rgmii_tx_clk_1,
                                self.dut.rgmii_rxd_1, self.dut.rgmii_rx_ctl_1, self.dut.rgmii_rx_clk_1)

        self.rgmii_2 = RgmiiPhy(self.dut.rgmii_txd_2, self.dut.rgmii_tx_ctl_2, self.dut.rgmii_tx_clk_2,
                                self.dut.rgmii_rxd_2, self.dut.rgmii_rx_ctl_2, self.dut.rgmii_rx_clk_2)

        self.rgmii_3 = RgmiiPhy(self.dut.rgmii_txd_3, self.dut.rgmii_tx_ctl_3, self.dut.rgmii_tx_clk_3,
                                self.dut.rgmii_rxd_3, self.dut.rgmii_rx_ctl_3, self.dut.rgmii_rx_clk_3)

        self.rgmii = [
            self.rgmii_0,
            self.rgmii_1,
            self.rgmii_2,
            self.rgmii_3
        ]

    async def cycle_reset(self):
        await self._cycle_reset(self.dut.u_dut.i_rst_100, self.dut.u_dut.i_clk_100)
        await self._cycle_reset(self.dut.u_dut.i_rst_125, self.dut.u_dut.i_clk_125)

    async def _cycle_reset(self, rst, clk):
        rst.setimmediatevalue(0)
        await RisingEdge(clk)
        await RisingEdge(clk)
        rst.value = 1
        await RisingEdge(clk)
        await RisingEdge(clk)
        rst.value = 0
        await RisingEdge(clk)
        await RisingEdge(clk)


@cocotb.test()
async def test_sanity(dut):
    tb = TB(dut)

    await tb.cycle_reset()

    src_macs = [RandMAC()._fix() for _ in range(4)]
    dst_macs = src_macs[1:] + [src_macs[0]]

    print(src_macs)
    print(dst_macs)

    # src_macs = [i for i in range(4)]
    # dst_macs = [i+0x100 for i in range(4)]

    frames = []

    for i, (src_mac, dst_mac) in enumerate(zip(src_macs, dst_macs)):
        frame = Ether(src=src_mac, dst=dst_mac) / f"Hello from port {i}"
        frames.append(GmiiFrame.from_payload(frame.build()))


    for _ in range(4):
        for i in range(4):
            await tb.rgmii[i].rx.send(frames[i])

    await Timer(10000, "ns")