module mac_wrapper #(
    parameter TARGET = "XILINX",
    parameter IODDR_STYLE = "IODDR",
    parameter CLOCK_INPUT_STYLE = "BUFR"  ,
    parameter USE_CLK90 = "TRUE" 
)(
    input   i_clk_100,
    input   i_rst_100,

    input   i_clk_125,
    input   i_clk_125_90,
    input   i_rst_125,

    axis_intf.MASTER    m_eth_axis,
    axis_intf.SLAVE     s_eth_axis,

    input  wire         rgmii_rx_clk,
    input  wire [3:0]   rgmii_rxd,
    input  wire         rgmii_rx_ctl,
    output wire         rgmii_tx_clk,
    output wire [3:0]   rgmii_txd,
    output wire         rgmii_tx_ctl,

    cpuif_intf.SLAVE    s_reg_cpuif
);

`ifdef ASSERTIONS
initial begin
    assert(m_eth_axis.DATA_WIDTH == s_eth_axis.DATA_WIDTH)
end
`endif

`define COUNTER(x) \
    logic [31:0] ``x``_counter;                                 \
    always @(posedge i_clk_100) begin                           \
        if (i_rst_100) begin                                    \
            ``x``_counter <= '0;                                \
        end else begin                                          \
            if (``x`` && ``x``_counter != '1) begin             \
                ``x``_counter <= ``x``_counter + 1;             \
            end                                                 \
            if (mac_hwif_out.``x``_count.``x``.swacc) begin     \
                ``x``_counter <= '0;                            \
            end                                                 \
        end                                                     \
    end                                                         \
    assign mac_hwif_in.``x``_count.``x``.next = ``x``_counter;  \


// mac regs

mac_regs_pkg::mac_regs__in_t mac_hwif_in;
mac_regs_pkg::mac_regs__out_t mac_hwif_out;


mac_regs u_mac_regs(
    .clk                    (i_clk_100),
    .rst                    (i_rst_100),

    .s_cpuif_req            (s_reg_cpuif.req),
    .s_cpuif_req_is_wr      (s_reg_cpuif.req_is_wr),
    .s_cpuif_addr           (s_reg_cpuif.addr[4:0]),
    .s_cpuif_wr_data        (s_reg_cpuif.wr_data),
    .s_cpuif_wr_biten       (s_reg_cpuif.wr_biten),
    .s_cpuif_req_stall_wr   (),
    .s_cpuif_req_stall_rd   (),
    .s_cpuif_rd_ack         (s_reg_cpuif.rd_ack),
    .s_cpuif_rd_err         (),
    .s_cpuif_rd_data        (s_reg_cpuif.rd_data),
    .s_cpuif_wr_ack         (s_reg_cpuif.wr_ack),
    .s_cpuif_wr_err         (),

    .hwif_in                (mac_hwif_in),
    .hwif_out               (mac_hwif_out)
);

logic tx_error_underflow;
logic tx_fifo_overflow;
logic tx_fifo_bad_frame;
logic tx_fifo_good_frame;
logic rx_error_bad_frame;
logic rx_error_bad_fcs;
logic rx_fifo_overflow;
logic rx_fifo_bad_frame;
logic rx_fifo_good_frame;


`COUNTER(tx_fifo_bad_frame);
`COUNTER(rx_fifo_bad_frame);
`COUNTER(rx_fifo_good_frame);
`COUNTER(rx_error_bad_frame);
`COUNTER(rx_error_bad_fcs);

assign mac_hwif_in.stats.tx_error_underflow.hwset = tx_error_underflow;
assign mac_hwif_in.stats.tx_fifo_overflow.hwset = tx_fifo_overflow;
assign mac_hwif_in.stats.tx_fifo_bad_frame.hwset = tx_fifo_bad_frame;
assign mac_hwif_in.stats.tx_fifo_good_frame.hwset = tx_fifo_good_frame;
assign mac_hwif_in.stats.rx_error_bad_frame.hwset = rx_error_bad_frame;
assign mac_hwif_in.stats.rx_fifo_overflow.hwset = rx_fifo_overflow;
assign mac_hwif_in.stats.rx_fifo_bad_frame.hwset = rx_fifo_bad_frame;
assign mac_hwif_in.stats.rx_fifo_good_frame.hwset = rx_fifo_good_frame;

logic [7:0] ifg_125;
logic tx_en_125;
logic rx_en_125;

// dumb flop
always @(posedge i_clk_125) begin
    ifg_125 <= mac_hwif_out.ctrl.ifg.value;
    tx_en_125 <= mac_hwif_out.ctrl.tx_en.value;
    rx_en_125 <= mac_hwif_out.ctrl.rx_en.value;
end


// eth mac
eth_mac_1g_rgmii_fifo #(
    .TARGET             (TARGET),
    .IODDR_STYLE        (IODDR_STYLE),
    .CLOCK_INPUT_STYLE  (CLOCK_INPUT_STYLE),
    .AXIS_DATA_WIDTH    (m_eth_axis.DATA_WIDTH),
    .USE_CLK90          (USE_CLK90)
) u_eth_mac_1g_rgmii_fifo (
    .gtx_clk            (i_clk_125),
    .gtx_clk90          (i_clk_125_90),
    .gtx_rst            (i_rst_125),
    .logic_clk          (i_clk_100),
    .logic_rst          (i_rst_100),

    .tx_axis_tdata      (s_eth_axis.tdata),
    .tx_axis_tkeep      (s_eth_axis.tkeep),
    .tx_axis_tvalid     (s_eth_axis.tvalid),
    .tx_axis_tready     (s_eth_axis.tready),
    .tx_axis_tlast      (s_eth_axis.tlast),
    .tx_axis_tuser      (s_eth_axis.tuser),

    .rx_axis_tdata      (m_eth_axis.tdata),
    .rx_axis_tkeep      (m_eth_axis.tkeep),
    .rx_axis_tvalid     (m_eth_axis.tvalid),
    .rx_axis_tready     (m_eth_axis.tready),
    .rx_axis_tlast      (m_eth_axis.tlast),
    .rx_axis_tuser      (m_eth_axis.tuser),

    .rgmii_rx_clk       (rgmii_rx_clk),
    .rgmii_rxd          (rgmii_rxd),
    .rgmii_rx_ctl       (rgmii_rx_ctl),
    .rgmii_tx_clk       (rgmii_tx_clk),
    .rgmii_txd          (rgmii_txd),
    .rgmii_tx_ctl       (rgmii_tx_ctl),


    .tx_error_underflow (tx_error_underflow),
    .tx_fifo_overflow   (tx_fifo_overflow),
    .tx_fifo_bad_frame  (tx_fifo_bad_frame),
    .tx_fifo_good_frame (tx_fifo_good_frame),
    .rx_error_bad_frame (rx_error_bad_frame),
    .rx_error_bad_fcs   (rx_error_bad_fcs),
    .rx_fifo_overflow   (rx_fifo_overflow),
    .rx_fifo_bad_frame  (rx_fifo_bad_frame),
    .rx_fifo_good_frame (rx_fifo_good_frame),
    .speed              (mac_hwif_in.ctrl.speed.next),

    .cfg_ifg            (ifg_125),
    .cfg_tx_enable      (tx_en_125),
    .cfg_rx_enable      (rx_en_125)
);


endmodule