module mac_separate(
    input core_clk,
    input core_rst,

    axis_intf.SLAVE s_axis,
    axis_intf.MASTER m_axis,

    output logic        o_mac_valid,
    output logic [47:0] o_src_mac,
    output logic [47:0] o_dst_mac
);

logic mac_valid_next;
logic [47:0] src_mac, src_mac_next;
logic [47:0] dst_mac, dst_mac_next;

logic [47:0] latched_dst_mac, latched_dst_mac_next;
logic [47:0] latched_src_mac, latched_src_mac_next;

// Separate the 96 bit mac address from the incoming packet.
// Input with is 64 bits, so we need 2 cycles. We don't modify
// the original packet, it just goes into a shift register

enum logic [1:0] {CYCLE_1, CYCLE_2, IDLE, INVALID} state, state_next;

always_ff @(posedge core_clk) begin
    if (core_rst) begin
        state <= CYCLE_1;
        src_mac <= '0;
        dst_mac <= '0;

        latched_src_mac <= '0;
        latched_dst_mac <= '0;

        o_mac_valid <= '0;
    end else begin
        state <= state_next;
        src_mac <= src_mac_next;
        dst_mac <= dst_mac_next;

        latched_src_mac <= latched_src_mac_next;
        latched_dst_mac <= latched_dst_mac_next;

        o_mac_valid <= mac_valid_next;
    end
end

function logic [47:0] get_dst_mac(input logic [63:0] data);
    return {
        data[0*8 +: 8],
        data[1*8 +: 8],
        data[2*8 +: 8],
        data[3*8 +: 8],
        data[4*8 +: 8],
        data[5*8 +: 8]
    };
endfunction

function [15:0] get_src_mac1(input logic [63:0] data);
    return {
        data[6*8 +: 8],
        data[7*8 +: 8]
    };
endfunction

function [31:0] get_src_mac2(input logic [63:0] data);
    return {
        data[0*8 +: 8],
        data[1*8 +: 8],
        data[2*8 +: 8],
        data[3*8 +: 8]
    };
endfunction

// only output MAC in the idle state, when the packet is output
assign o_src_mac = (m_axis.tvalid) ? latched_src_mac : '0;
assign o_dst_mac = (m_axis.tvalid) ? latched_dst_mac : '0;

always_comb begin
    state_next = state;

    src_mac_next = src_mac;
    src_mac_next = src_mac;

    latched_dst_mac_next = latched_dst_mac;
    latched_dst_mac_next = latched_dst_mac;

    mac_valid_next = '0;

    case (state)
        CYCLE_1: begin
            if (s_axis.tready & s_axis.tvalid) begin
                dst_mac_next = get_dst_mac(s_axis.tdata);
                src_mac_next[47:32] = get_src_mac1(s_axis.tdata);
                state_next = CYCLE_2;
            end
        end

        CYCLE_2: begin
            if (s_axis.tready & s_axis.tvalid) begin
                src_mac_next[31:0] = get_src_mac2(s_axis.tdata);
                state_next = IDLE;

                mac_valid_next = '1;

                latched_dst_mac_next = dst_mac;
                latched_src_mac_next = src_mac_next; // save one cycle here
            end
        end

        IDLE: begin
            if (m_axis.tvalid & m_axis.tready & s_axis.tlast) begin
                state_next = CYCLE_1;
            end
        end

        default: begin
            $error("Invalid case in mac separate");
        end
    endcase
end


// packet shift register
axis_pipeline_register_wrapper #(
    .REG_TYPE(1),
    .LENGTH(2)
) u_axis_pipe (
    .clk        (core_clk),
    .rst        (core_rst),

    .s_axis     (s_axis),
    .m_axis     (m_axis)
);


endmodule