module axis_output_demux #(
    parameter M_COUNT = 1
) (
    input       i_clk,
    input       i_rst,

    axis_intf.SLAVE         s_axis,
    axis_intf.MASTER        m_axis[M_COUNT],
    axis_intf.MASTER        m_axis_broadcast[M_COUNT]
);

`ifdef ASSERTIONS
initial begin
    if s_axis.DATA_WIDTH != m_axis[0].DATA_WIDTH) begin
        $error("demux input and output widths are not equal")!
    end
end
`endif

localparam DATA_WIDTH = s_axis.DATA_WIDTH;
localparam USER_WIDTH = 1;
localparam KEEP_WIDTH = ((DATA_WIDTH+7)/8);

logic [M_COUNT*DATA_WIDTH-1:0]  m_axis_tdata;
logic [M_COUNT*KEEP_WIDTH-1:0]  m_axis_tkeep;
logic [M_COUNT-1:0]             m_axis_tvalid;
logic [M_COUNT-1:0]             m_axis_tready;
logic [M_COUNT-1:0]             m_axis_tlast;
logic [M_COUNT*USER_WIDTH-1:0]  m_axis_tuser;

logic [M_COUNT*DATA_WIDTH-1:0]  m_axis_broadcast_tdata;
logic [M_COUNT*KEEP_WIDTH-1:0]  m_axis_broadcast_tkeep;
logic [M_COUNT-1:0]             m_axis_broadcast_tvalid;
logic [M_COUNT-1:0]             m_axis_broadcast_tready;
logic [M_COUNT-1:0]             m_axis_broadcast_tlast;
logic [M_COUNT*USER_WIDTH-1:0]  m_axis_broadcast_tuser;

generate
    for (genvar i = 0; i < M_COUNT; i++) begin
        assign m_axis[i].tdata = m_axis_tdata[i*DATA_WIDTH+:DATA_WIDTH];
        assign m_axis[i].tkeep = m_axis_tkeep[i*KEEP_WIDTH+:KEEP_WIDTH];
        assign m_axis[i].tvalid = m_axis_tvalid[i];
        assign m_axis_tready[i] = m_axis[i].tready;
        assign m_axis[i].tlast = m_axis_tlast[i] ;
        assign m_axis[i].tuser = m_axis_tuser[i*USER_WIDTH+:USER_WIDTH];
    end
endgenerate

generate
    for (genvar i = 0; i < M_COUNT; i++) begin
        assign m_axis_broadcast[i].tdata = m_axis_broadcast_tdata[i*DATA_WIDTH+:DATA_WIDTH];
        assign m_axis_broadcast[i].tkeep = m_axis_broadcast_tkeep[i*KEEP_WIDTH+:KEEP_WIDTH];
        assign m_axis_broadcast[i].tvalid = m_axis_broadcast_tvalid[i];
        assign m_axis_broadcast_tready[i] = m_axis_broadcast[i].tready;
        assign m_axis_broadcast[i].tlast = m_axis_broadcast_tlast[i] ;
        assign m_axis_broadcast[i].tuser = m_axis_broadcast_tuser[i*USER_WIDTH+:USER_WIDTH];
    end
endgenerate

localparam S_ID_WIDTH = $clog2(M_COUNT);


logic [DATA_WIDTH-1:0]  demux_axis_tdata;
logic [KEEP_WIDTH-1:0]  demux_axis_tkeep;
logic [7:0]             demux_axis_tdest;
logic                   demux_axis_tvalid;
logic                   demux_axis_tready;
logic                   demux_axis_tlast;
logic [USER_WIDTH-1:0]  demux_axis_tuser;

logic [DATA_WIDTH-1:0]  broadcast_axis_tdata;
logic [KEEP_WIDTH-1:0]  broadcast_axis_tkeep;
logic [7:0]             broadcast_axis_tdest;
logic                   broadcast_axis_tvalid;
logic                   broadcast_axis_tready;
logic                   broadcast_axis_tlast;
logic [USER_WIDTH-1:0]  broadcast_axis_tuser;


// demux between broadcast and single port
axis_demux #(
    .M_COUNT(2),
    .DATA_WIDTH(DATA_WIDTH),
    .DEST_ENABLE(1),
    .TDEST_ROUTE(0),
    .S_DEST_WIDTH(8)
) u_broadcast_demux (
    .clk            (i_clk),
    .rst            (i_rst),

    .s_axis_tdata   (s_axis.tdata),
    .s_axis_tkeep   (s_axis.tkeep),
    .s_axis_tvalid  (s_axis.tvalid),
    .s_axis_tready  (s_axis.tready),
    .s_axis_tlast   (s_axis.tlast),
    .s_axis_tid     (s_axis.tid),
    .s_axis_tdest   (s_axis.tdest),
    .s_axis_tuser   (s_axis.tuser),

    .m_axis_tdata   ({broadcast_axis_tdata, demux_axis_tdata}),
    .m_axis_tkeep   ({broadcast_axis_tkeep, demux_axis_tkeep}),
    .m_axis_tvalid  ({broadcast_axis_tvalid, demux_axis_tvalid}),
    .m_axis_tready  ({broadcast_axis_tready, demux_axis_tready}),
    .m_axis_tlast   ({broadcast_axis_tlast, demux_axis_tlast}),
    .m_axis_tid     (),
    .m_axis_tdest   ({broadcast_axis_tdest, demux_axis_tdest}),
    .m_axis_tuser   ({broadcast_axis_tuser, demux_axis_tuser}),

    .enable         ('1),
    .drop           ('0),
    .select         (s_axis.tdest == '1)
);

logic [7:0] demux_test_altered;
assign demux_test_altered = {demux_axis_tdest[S_ID_WIDTH-1:0], 6'h0};

// demux from single port to all ports
axis_demux #(
    .M_COUNT(M_COUNT),
    .DATA_WIDTH(DATA_WIDTH),
    .DEST_ENABLE(1),
    .TDEST_ROUTE(1),
    .S_DEST_WIDTH(8)
) u_axis_demux (
    .clk            (i_clk),
    .rst            (i_rst),

    .s_axis_tdata   (demux_axis_tdata),
    .s_axis_tkeep   (demux_axis_tkeep),
    .s_axis_tvalid  (demux_axis_tvalid),
    .s_axis_tready  (demux_axis_tready),
    .s_axis_tlast   (demux_axis_tlast),
    .s_axis_tid     (),
    .s_axis_tdest   (demux_test_altered),
    .s_axis_tuser   (demux_axis_tuser),

    .m_axis_tdata   (m_axis_tdata),
    .m_axis_tkeep   (m_axis_tkeep),
    .m_axis_tvalid  (m_axis_tvalid),
    .m_axis_tready  (m_axis_tready),
    .m_axis_tlast   (m_axis_tlast),
    .m_axis_tid     (),
    .m_axis_tdest   (),
    .m_axis_tuser   (m_axis_tuser),

    .enable         ('1),
    .drop           ('0),
    .select         ('0)
);

// broadcast to all ports
axis_broadcast #(
    .M_COUNT(M_COUNT),
    .DATA_WIDTH(DATA_WIDTH)
) u_axis_broadcast (
    .clk            (i_clk),
    .rst            (i_rst),

    .s_axis_tdata   (broadcast_axis_tdata),
    .s_axis_tkeep   (broadcast_axis_tkeep),
    .s_axis_tvalid  (broadcast_axis_tvalid),
    .s_axis_tready  (broadcast_axis_tready),
    .s_axis_tlast   (broadcast_axis_tlast),
    .s_axis_tid     ('0),
    .s_axis_tdest   ('0),
    .s_axis_tuser   (broadcast_axis_tuser),

    .m_axis_tdata   (m_axis_broadcast_tdata),
    .m_axis_tkeep   (m_axis_broadcast_tkeep),
    .m_axis_tvalid  (m_axis_broadcast_tvalid),
    .m_axis_tready  (m_axis_broadcast_tready),
    .m_axis_tlast   (m_axis_broadcast_tlast),
    .m_axis_tid     (),
    .m_axis_tdest   (),
    .m_axis_tuser   (m_axis_broadcast_tuser)
);


endmodule