import cocotb 

from cocotb.clock import Clock 
from cocotb.triggers import Timer, RisingEdge, FallingEdge

from scapy.layers.inet import Ether

from scapy.volatile import RandMAC

from cocotbext.axi import AxiStreamBus, AxiStreamSink, AxiStreamSource, AxiLiteMaster, AxiLiteBus

import logging

CLK_PERIOD_CORE = 10 # also 100MHz for the time being

class TB:
    def __init__(self, dut):
        self.dut = dut

        self.log = logging.getLogger("cocotb.tb")
        self.log.setLevel(logging.INFO)

        cocotb.start_soon(Clock(dut.u_dut.core_clk, CLK_PERIOD_CORE, units="ns").start())

        self.m_axis = AxiStreamSink(AxiStreamBus.from_prefix(dut.m_axis, ""), dut.u_dut.core_clk, dut.u_dut.core_rst)
        self.s_axis = AxiStreamSource(AxiStreamBus.from_prefix(dut.s_axis, ""), dut.u_dut.core_clk, dut.u_dut.core_rst)

    async def cycle_reset(self):
        await self._cycle_reset(self.dut.u_dut.core_rst, self.dut.u_dut.core_clk)

    async def _cycle_reset(self, rst, clk):
        rst.setimmediatevalue(0)
        await RisingEdge(clk)
        await RisingEdge(clk)
        rst.value = 1
        await RisingEdge(clk)
        await RisingEdge(clk)
        rst.value = 0
        await RisingEdge(clk)
        await RisingEdge(clk)


@cocotb.test()
async def test_sanity(dut):
    tb = TB(dut)

    await tb.cycle_reset()

    for _ in range(16):
        blank_ether = Ether(src=RandMAC()._fix(), dst=RandMAC()._fix()) / "Padding text"

        await tb.s_axis.send(blank_ether.build())

        await RisingEdge(tb.m_axis.bus.tvalid)
        separate_src_mac = tb.dut.u_dut.o_src_mac.value.integer
        separate_dst_mac = tb.dut.u_dut.o_dst_mac.value.integer

        print(f"{separate_src_mac:x}")
        print(f"{separate_dst_mac:x}")

        original_src_mac = int(blank_ether.src.replace(":", ""), 16)
        original_dst_mac = int(blank_ether.dst.replace(":", ""), 16)

        assert separate_src_mac == original_src_mac
        assert separate_dst_mac == original_dst_mac


        frame = await tb.m_axis.read()
        frame = bytes(frame)

        assert frame == blank_ether.build()

    await Timer(1000, "ns")