import cocotb 

from cocotb.clock import Clock 
from cocotb.triggers import Timer, RisingEdge, FallingEdge

from cocotbext.axi import AxiStreamBus, AxiStreamSink, AxiStreamSource, AxiLiteMaster, AxiLiteBus, AxiStreamFrame

import logging

CLK_PERIOD_ETH = 10
CLK_PERIOD_CORE = 10 # also 100MHz for the time being

class TB:
    def __init__(self, dut):
        self.dut = dut

        self.log = logging.getLogger("cocotb.tb")
        self.log.setLevel(logging.INFO)

        cocotb.start_soon(Clock(dut.u_dut.eth_clk, CLK_PERIOD_ETH, units="ns").start())
        cocotb.start_soon(Clock(dut.u_dut.core_clk, CLK_PERIOD_CORE, units="ns").start())

        self.s_axis = AxiStreamSource(AxiStreamBus.from_prefix(dut.s_axis, ""), dut.u_dut.core_clk, dut.u_dut.core_rst)
        self.m_axis = [
            AxiStreamSink(AxiStreamBus.from_prefix(dut.m_axis[i], ""), dut.u_dut.eth_clk, dut.u_dut.eth_rst)
            for i in range(4)
        ]

    async def cycle_reset(self):
        await self._cycle_reset(self.dut.u_dut.core_rst, self.dut.u_dut.core_clk)
        await self._cycle_reset(self.dut.u_dut.eth_rst, self.dut.u_dut.eth_clk)

    async def _cycle_reset(self, rst, clk):
        rst.setimmediatevalue(0)
        await RisingEdge(clk)
        await RisingEdge(clk)
        rst.value = 1
        await RisingEdge(clk)
        await RisingEdge(clk)
        rst.value = 0
        await RisingEdge(clk)
        await RisingEdge(clk)

@cocotb.test()
async def test_sanity(dut):
    tb = TB(dut)

    await tb.cycle_reset()

    tx_strs = [f"Hello, world! Sending this from port {i}".encode("ascii") for i in range(4)]


    for i in range(4):
        frame = AxiStreamFrame(tx_strs[i], tdest=i)
        frame.normalize()
        await tb.s_axis.send(frame)

    for i in range(4):
        val = await tb.m_axis[i].recv()
        assert val.tdata == tx_strs[i]